import path from 'node:path';
import { compile as platformCompile } from '@lwc/sfdc-lwc-compiler';
import { Workspace, LogLevel } from '../../types/index.js';
import { readModule } from '../util/fileUtil.js';
/**
 * Fetch files and source related to the modulePath that is being hot loaded.
 * @param {DevServerContext} ctx DevServer object for stateful information
 * @param {string} name Name of the component being compiled
 * @param {string} namespace The namespace of the component
 * @param {string} modulePath The formatted path for the file being compiled.
 * @returns
 */
function getBundleFiles(ctx, name, namespace, modulePath) {
    const moduleDir = `${namespace}/${name}`;
    const moduleFilePath = ctx.moduleGraph.getFileByModulePath(modulePath);
    const fileEntry = path.relative(moduleDir, modulePath);
    // It can be assumed that the module file exists and the source can be read
    const src = readModule(moduleFilePath);
    return { [fileEntry]: src };
}
function transformOptionsToPlatformCompileConfig(ctx, transformOptions, modulePath) {
    const { enableDynamicComponents, enableLightningWebSecurityTransforms, experimentalComplexExpressions, apiVersion, name, namespace, experimentalDynamicComponent, outputConfig, } = transformOptions;
    const options = {
        // variable options
        allowDynamicComponents: enableDynamicComponents,
        enableLightningWebSecurityTransforms: enableLightningWebSecurityTransforms,
        enableTemplateExpressions: experimentalComplexExpressions,
        apiVersion: apiVersion,
        // Standard options
        disableProcessNodeEnv: true,
        dynamicImport: 'allowed',
        forceNamedExports: true,
        strictCSSTokenCollection: true,
        // Enable HMR always in dev server
        enableHmr: true,
    };
    const getBundleType = () => {
        if (ctx.config.workspace === Workspace.Internal || ctx.config.workspace === Workspace.Mrt) {
            return 'internal';
        }
        return 'platform';
    };
    const bundle = {
        type: getBundleType(),
        name: name,
        namespace: namespace ?? 'c',
        files: getBundleFiles(ctx, name, namespace ?? '', modulePath),
        options,
    };
    const compilerConfigs = [
        {
            outputConfig: { compat: false, minify: false, sourcemap: !!outputConfig?.sourcemap },
            experimentalDynamicComponent: experimentalDynamicComponent,
        },
    ];
    return {
        bundle,
        compilerConfigs,
    };
}
export async function compile(ctx, modulePath, options, logger) {
    const compileConfig = transformOptionsToPlatformCompileConfig(ctx, options, modulePath);
    const result = await platformCompile(compileConfig);
    if (!result.success) {
        logger.log(LogLevel.error, `Failed to compile ${modulePath}. \n\tDiagnostics: ${JSON.stringify(result.diagnostics)}`);
        return { code: undefined, warnings: [result.diagnostics.toString()] };
    }
    return {
        code: result.results?.[0].code,
        warnings: result.diagnostics.length ? [result.diagnostics.toString()] : [],
    };
}
//# sourceMappingURL=lexBundler.js.map