import path from 'node:path';
import { compile } from '../bundler/lexBundler.js';
import { LogLevel } from '../../types/serverConfig.js';
const ALLOWED_OUTBOUND_DATA_TYPES = new Set([
    'update',
    'module-update',
    'module-delete',
    'error',
]);
const ALLOWED_INBOUND_DATA_TYPES = new Set(['fetch', 'register']);
const SUPPORTED_FETCH_EXTENSIONS = new Set(['.html', '.css', '.js']);
/**
 * Class representing a client that has connected to the dev server.
 * This class represents a client that is connected and ready to receive hot module updates.
 * After the hmr-client is connected, all communication between itself and lwc-dev-server are handled
 * by this class.
 */
export class Client {
    constructor(clientId, url, target, connection, ctx) {
        this.activePaths = new Map();
        this.clientId = clientId;
        this.url = url;
        this.target = target;
        this.connection = connection;
        this.connection.receive(this.messageCallback.bind(this));
        this.ctx = ctx;
    }
    addActivePaths(paths) {
        paths.forEach(({ modulePath, compileOptions }) => {
            const existing = this.activePaths.get(modulePath);
            if (!existing) {
                this.activePaths.set(modulePath, compileOptions);
            }
        });
    }
    removeStalePaths(paths) {
        paths.forEach((p) => this.activePaths.delete(p));
    }
    getCompileOptions(path) {
        return this.activePaths.get(path);
    }
    send(data) {
        if (!ALLOWED_OUTBOUND_DATA_TYPES.has(data.type)) {
            this.ctx.logger.log(LogLevel.debug, `[LWC Dev Server] Blocking unexpected type of data being sent from server to hmr-client '${data.type}'`);
            return;
        }
        this.connection.send(data);
    }
    async messageCallback(data) {
        if (!ALLOWED_INBOUND_DATA_TYPES.has(data.type)) {
            this.ctx.logger.log(LogLevel.debug, `[LWC Dev Server] Blocking unexpected type of data received from hmr-client '${data.type}'`);
            return;
        }
        const { ctx: { logger, moduleGraph }, } = this;
        switch (data.type) {
            // In coming hot module
            case 'fetch': {
                const { data: { modulePath }, } = data;
                const moduleFilePath = moduleGraph.getFileByModulePath(modulePath);
                if (!moduleFilePath) {
                    const error = {
                        type: 'error',
                        data: {
                            message: `Fetch of ${modulePath} failed as it does not exist in dev server workspace`,
                        },
                    };
                    this.connection.send(error);
                }
                switch (this.target) {
                    case 'LEX': {
                        const compileOptions = this.getCompileOptions(modulePath);
                        if (!compileOptions) {
                            logger.log(LogLevel.debug, `Found no compile option for module path: ${modulePath}, check the __lwc_hmr_hot.register() call.`);
                            break;
                        }
                        logger.log(LogLevel.debug, `[LWC Dev Server] Compiling ${modulePath}`);
                        const result = await compile(this.ctx, modulePath, compileOptions, logger);
                        if (!result.code) {
                            logger.log(LogLevel.error, `Compiling ${moduleFilePath} failed and there are compiler warnings: ${JSON.stringify(result.warnings)}`);
                            break;
                        }
                        logger.log(LogLevel.debug, `[LWC Dev Server] Sending an updated ${modulePath}`);
                        this.connection.send({
                            type: 'module-update',
                            data: [{ modulePath, src: result.code }],
                        });
                        break;
                    }
                    default:
                        logger.log(LogLevel.verbose, `Client target ${this.target} not supported currently.`);
                        break;
                }
                break;
            }
            case 'register': {
                const moduleRegistration = data;
                const staleModules = moduleGraph.registerActivePaths(moduleRegistration);
                this.addActivePaths(moduleRegistration.data.modules.map(({ modulePath, compileOptions, hash: _hash }) => {
                    return { modulePath, compileOptions };
                }));
                const availableUpdates = {
                    type: 'update',
                    data: staleModules
                        // This is a hack until we add support for javascript hot module reloading
                        .filter((modulePath) => SUPPORTED_FETCH_EXTENSIONS.has(path.extname(modulePath)))
                        .map((modulePath) => {
                        return { modulePath };
                    }),
                };
                this.send(availableUpdates);
                break;
            }
        }
    }
    close() {
        this.connection.close();
    }
}
/**
 * Class to manage all connected clients.
 * This serves as a manager of clients with routines to broadcast updates, reset clients.
 */
export class ClientPool {
    constructor() {
        this.socketToClientMap = new Map();
        this.clientToSocketMap = new Map();
    }
    addClient(ws, client) {
        this.socketToClientMap.set(ws, client);
        this.clientToSocketMap.set(client, ws);
    }
    removeClient(client) {
        const socket = this.clientToSocketMap.get(client);
        if (socket) {
            this.socketToClientMap.delete(socket);
            this.clientToSocketMap.delete(client);
        }
    }
    /**
     * Broad case to all clients of an available module update.
     * @param message
     */
    sendModuleUpdateBroadcast(message) {
        for (const client of this.clientToSocketMap.keys()) {
            client.send(message);
        }
    }
    reset() {
        for (const client of this.clientToSocketMap.keys()) {
            client.close();
        }
        this.socketToClientMap.clear();
        this.clientToSocketMap.clear();
    }
    size() {
        return this.clientToSocketMap.size;
    }
}
//# sourceMappingURL=client.js.map