import { LogLevel, DevServerContext } from '../types/index.js';
import { IGNORING_PREINIT_RESPONSES, UNRECOGNIZED_MESSAGE_TYPE } from '../labels.js';
import { startWatch } from './watcher/index.js';
import { startWebSocketServer } from './connection/web-socket-server.js';
import { ModuleGraph } from './moduleGraph/index.js';
import ConsoleLogger from './consoleLogger.js';
import { Client, ClientPool } from './connection/client.js';
import { Connection } from './connection/connection.js';
/**
 * This class represents the central place where the LWC dev server coordination occurs.
 *
 * ModuleGraph -> Is responsible for discovering modules and tracking changes to them
 * Connection -> An authenticated connection between hmr-client and dev server
 * Client -> A stateful object that tracks a hmr-client connected to the dev server
 *
 * This class also manages resources that will need clean up, like the file watcher, WebSockets.
 */
export class LWCServer {
    constructor(config, wss, logger) {
        this.clientPool = new ClientPool();
        // Dev Server to manage critical assets, allows them to be closed and avoid memory leaks
        this.wss = wss;
        this.watcher = startWatch(config.paths);
        this.init();
        this.context = new DevServerContext(config, logger, new ModuleGraph(config, logger, this.watcher, {
            moduleUpdateHandler: this.moduleUpdateHandler.bind(this),
        }));
    }
    /**
     * Initialize the WebSocketServer by adding message handlers for connections lifecycle events.
     */
    init() {
        // New connection handler
        this.wss.on('connection', (ws) => {
            // New clients have to call the init as the very first message and after that the
            // communication will be moved off to be handled by the Client class
            ws.addEventListener('message', this.newClientMessageCallback.bind(this), {
                once: true,
            });
            ws.addEventListener('close', this.clientCloseHandler.bind(this), { once: true });
            ws.addEventListener('error', this.clientCloseHandler.bind(this), { once: true });
        });
        this.watcher.on('error', (event) => {
            this.context.logger.log(LogLevel.error, '[LWC Dev Server] Encountered error while watching files');
            this.context.logger.log(LogLevel.debug, `[LWC Dev Server] File watcher error: ${event.message}`);
            this.stopServer();
        });
    }
    stopServer() {
        this.context.logger.log(LogLevel.info, '[LWC Dev Server] Stopping the dev server');
        this.watcher.close();
        this.clientPool.reset();
        this.wss.close();
        this.context.logger.log(LogLevel.info, '[LWC Dev Server] Succesfully stopped dev server');
        process.exit(0);
    }
    moduleUpdateHandler(modulePath) {
        const hotModule = { type: 'update', data: [{ modulePath }] };
        this.clientPool.sendModuleUpdateBroadcast(hotModule);
    }
    /**
     * A one time message handler for new clients.
     * This method is responsible for authenticating the client connection and once authenticated,
     * the communication is handed over to the Client object.
     */
    newClientMessageCallback({ data: rawData, target: socket }) {
        const { context: { logger }, } = this;
        let payload;
        if (typeof rawData === 'string') {
            try {
                payload = JSON.parse(rawData);
            }
            catch (e) {
                logger.log(LogLevel.error, 'Failed to read message sent by hmr-client. Client connected from ' +
                    socket.url);
                socket.close();
            }
        }
        const data = payload;
        if (data.type == undefined) {
            logger.log(LogLevel.error, UNRECOGNIZED_MESSAGE_TYPE);
            socket.close();
            return;
        }
        switch (data.type) {
            case 'init': {
                const { data: { clientId, url, target, auth }, } = data;
                if (this.clientPool.size() === this.context.config.maxClientCount) {
                    logger.log(LogLevel.error, `[LWC Dev Server] Max connection limit(${this.context.config.maxClientCount}) reached, ignoring new requests until existing connections are closed`);
                    socket.close();
                    return;
                }
                const connection = new Connection(socket, logger);
                const client = new Client(clientId, url, target, connection, this.context);
                this.clientPool.addClient(socket, client);
                logger.log(LogLevel.info, `[LWC Dev Server] New connection for url: ${url} and target: ${target} registered. Client ID: ${clientId}`);
                // TODO: The server should decode the auth challenge with its session key and return the challenge phrase
                if (auth) {
                    const authResponse = {
                        type: 'auth-response',
                        data: {
                            sessionToken: 'tango-charlie:' + auth.data.sessionTokenCipherText,
                        },
                    };
                    client.send(authResponse);
                }
                break;
            }
            default: {
                logger.log(LogLevel.warn, IGNORING_PREINIT_RESPONSES);
                socket.close();
                break;
            }
        }
    }
    clientCloseHandler({ target: socket }) {
        const client = this.clientPool.socketToClientMap.get(socket);
        if (!client) {
            // Should never get here, but if we do then forcefully shut the socket
            this.context.logger.log(LogLevel.error, '[LWC Dev Server] Unrecognized client, shutting down the socket');
            socket.close();
            return;
        }
        // Else let the client handle the cleanup
        client.close();
        this.clientPool.removeClient(client);
        this.context.logger.log(LogLevel.info, `[LWC Dev Server] Closing client. Client ID: ${client.clientId}`);
    }
}
/**
 * Initialize an LWC Dev Server with the specified server configuration.
 * @returns Returns an initialized LWC dev server.
 */
export async function startLwcDevServer(config) {
    const logger = new ConsoleLogger(config.logLevel);
    const wss = startWebSocketServer(config, logger);
    return new LWCServer(config, wss, logger);
}
//# sourceMappingURL=index.js.map