import fs from 'fs';
import path from 'path';
import { LogLevel } from '../../types/index.js';
import { getModuleHash } from '../util/fileUtil.js';
import { getWorkspaceScanner } from './workspaceScanner/index.js';
export class ModuleGraph {
    constructor(config, logger, watcher, moduleChangeCallbacks) {
        /**
         * All LWC modules and their parts with absolute file paths, for the current workspace.
         */
        this.moduleMap = [];
        /**
         * All modules paths that are being used by the various client connections
         */
        this.activeModulePaths = [];
        // Mapping from <absolute file path> => "<namespace>/<componentname>/component.js"
        this.fileToModuleMap = Object.create(null);
        // Mapping from "<namespace>/<componentname>/component.js" => { moduleFilePath, moduleSourceHash }
        this.modulePathToModuleInfo = Object.create(null);
        this.watcher = watcher;
        this.logger = logger;
        this.moduleChangeCallbacks = moduleChangeCallbacks;
        this.workspaceScanner = getWorkspaceScanner(config.workspace, config.paths, logger);
        this.init();
    }
    init() {
        this.moduleMap = this.workspaceScanner.getModulesMap();
        if (this.moduleMap.length) {
            this.logger.log(LogLevel.debug, `[Module Graph] Found ${this.moduleMap.length} LWC components in the current workspace.`);
        }
        else {
            this.logger.log(LogLevel.debug, `[Module Graph] Found no LWC components in the current workspace, check your settings in lwc.config.json.`);
        }
        this.moduleMap.forEach(({ moduleParts }) => {
            Object.assign(this.modulePathToModuleInfo, moduleParts);
            for (const [modulePath, moduleInfo] of Object.entries(moduleParts)) {
                this.fileToModuleMap[moduleInfo.moduleFilePath] = modulePath;
            }
        });
        this.watcher.on('change', this.fileChangeHandler.bind(this));
    }
    getModuleByFile(path) {
        return this.fileToModuleMap[path];
    }
    getFileByModulePath(modulePath) {
        return this.modulePathToModuleInfo[modulePath]?.moduleFilePath;
    }
    /**
     * Register the active paths that the client is interested in
     * @param moduleRegistration
     * @returns Modules paths that are determined to be stale wrt the content monitored by the dev server
     */
    registerActivePaths(moduleRegistration) {
        const { data: { modules: activeModules }, } = moduleRegistration;
        const staleModules = [];
        const unTrackedModules = [];
        activeModules.forEach((mod) => {
            const { modulePath, hash: _hash } = mod;
            if (modulePath.trim()) {
                const moduleInfo = this.modulePathToModuleInfo[modulePath];
                if (!moduleInfo) {
                    unTrackedModules.push(modulePath);
                }
                else {
                    this.activeModulePaths.push(modulePath);
                    if (moduleInfo.moduleSourceHash !== _hash) {
                        staleModules.push(modulePath);
                    }
                }
            }
        });
        unTrackedModules.length &&
            this.logger.log(LogLevel.debug, `[Module Graph] Following modules are unknown to dev server: \n\t ${unTrackedModules.join(',\n\t')}`);
        staleModules.length &&
            this.logger.log(LogLevel.debug, `[Module Graph] Following modules have a newer version: \n\t ${staleModules.join(',\n\t')}`);
        return staleModules;
    }
    fileChangeHandler(filePath) {
        const changedFile = path.resolve(filePath);
        if (fs.lstatSync(changedFile).isFile()) {
            const modulePath = this.getModuleByFile(changedFile) ?? changedFile;
            if (modulePath) {
                const moduleInfo = this.modulePathToModuleInfo[modulePath];
                const newModuleHash = getModuleHash(changedFile);
                if (moduleInfo.moduleSourceHash === newModuleHash) {
                    return;
                }
                this.logger.log(LogLevel.info, `[LWC Dev Server] File '${modulePath}' modified - updating.`);
                moduleInfo.moduleSourceHash = newModuleHash;
                this.moduleChangeCallbacks.moduleUpdateHandler(modulePath);
            }
        }
    }
}
//# sourceMappingURL=index.js.map