import { existsSync, readdirSync } from 'node:fs';
import path from 'node:path';
import { getModuleHash } from '../../util/fileUtil.js';
/**
 * Is the file a valid LWC component part?
 * @param {string} file
 * @param {string} componentName Component name
 * @returns {boolean}
 */
function isValidLwcFile(file, componentName) {
    const { name: fileName, dir: dirPath, ext } = path.parse(file);
    const dir = path.basename(dirPath);
    if (['.ts', '.js', '.html', '.css'].includes(ext))
        return true;
    if (dir === '__docs__' && (ext === '.md' || ext === '.auradoc') && fileName === componentName)
        return true;
    if (dir === componentName && ext === '.svg' && fileName === componentName)
        return true;
    return false;
}
/**
 * Should the directory be processed as a valid LWC module part?
 */
function shouldProcessDirectory(file) {
    const name = path.basename(file);
    return !name.startsWith('__') || name === '__docs__';
}
/**
 * Get valid LWC files recursively from a directory
 * @param {string} dir Component Directory
 * @param {string} name Component name
 * @returns {Promise<string[]>} valid LWC file paths
 */
function getFiles(dir, name) {
    if (!shouldProcessDirectory(dir)) {
        return [];
    }
    const dirents = readdirSync(dir, { withFileTypes: true });
    return dirents.flatMap((dirent) => {
        const file = path.resolve(dir, dirent.name);
        if (dirent.isDirectory()) {
            return getFiles(file, name);
        }
        return isValidLwcFile(file, name) ? [file] : [];
    });
}
/**
 * Verify if a given directory represents an LWC component directory by checking if a valid entry file exists.
 * @param entryPoint The directory to test.
 * @param componentName The component name.
 * @returns {boolean} true if directory has an entry file.
 */
function isValidEntryPoint(entryPoint, componentName) {
    const jsEntryFile = path.join(entryPoint, componentName + '.js');
    const tsEntryFile = path.join(entryPoint, componentName + '.ts');
    const cssOnlyModuleEntryFile = path.join(entryPoint, componentName + '.css');
    return existsSync(jsEntryFile) || existsSync(tsEntryFile) || existsSync(cssOnlyModuleEntryFile);
}
/**
 * Get all parts of an LWC component.
 * @param componentDir A directory representing an LWC Component.
 * @returns A map representing all valid parts of the LWC with the short path as the key and full
 *  canonical path as value.
 */
export function getComponentParts(componentDir, namespace) {
    if (!existsSync(componentDir)) {
        throw new Error('The LWC component directory does not exist.');
    }
    const name = path.basename(componentDir);
    if (!isValidEntryPoint(componentDir, name)) {
        // Not an LWC directory, return
        return undefined;
    }
    const files = getFiles(componentDir, name);
    return Object.fromEntries(
    // Note: The file separator in the key should match that used by the
    // @lwc/dev-server-plugin-lex and not the host system where the dev server runs
    files.map((file) => {
        return [
            `${namespace}/${name}/${path.relative(componentDir, file)}`,
            {
                moduleFilePath: file,
                moduleSourceHash: getModuleHash(file),
            },
        ];
    }));
}
//# sourceMappingURL=fileReader.js.map